#!/usr/bin/perl
#----------------------------------------------------------------------
# pkirisCsrMulti.pl                                  RedIRIS - 20060803
#
# Genera un XML con un conjunto de CSRs preparado para
# volcarlo en la pkIRISGrid CA. El programa genera:
#
# - Ficheros de CSRs y de claves privadas asociadas
# - Fichero XML con todas las CSRs y datos del administrador
# - Fichero con los pines asociados a cada certificado para su
#   uso futuro por si se desea revocar algn certificado
#----------------------------------------------------------------------
#

use vars qw/ %opt /;
use Getopt::Std;
use Cwd;

# Datos generales de la aplicacin
my %pkirisData = (
  'myname'         => "pkirisCsrMulti.pl",
  'version'        => "20060804-0.4.0",
  'iFQDNsFileName' => "FQDN.txt",  # Fichero con los FQDNs
  'iComment'       => "#",         # Marca de comentario en fichero con FQDNs
  'oDir'           => "./salida",  # Directorio de salida
  'oCsrXmlFileName'=> "pkirisCsrMulti-csrData.xml",
  'oPinFileName'   => "pkirisCsrMulti-csrPin.txt",
  'openssl'        => "/usr/bin/openssl",
  'opensslCf'      => "openssl.cf",
  'serviceSep'     => "---",       # cadena mostrada en lugar de "/" cuando tratemos un servicio
  'pin'            => "random",    # random: aleatorio para cada CSR, o valor: el mismo para todos
  'bits'           => "2048",      # nmero de bits
);

# Datos del administrador
%adminData = (
  'name'        => 'Javier',
  'surname'     => 'Masa Marin',
  'passwd'      => '0..70BdKPb.8s',
  'telephone'   => '+34 955056623',
  'mail'        => 'javier.masa@rediris.es',
  'domain'      => 'rediris.es',
);

# Datos relacionados con la salida XML
my %pkirisXmlData = (
  'top'			=> "<pkiris type='csr_multi' v='1.0'>\n",
  'bottom'		=> "</pkiris>",
);

getopts ( "a:b:df:ho:p:v", \%opt ) or showHelp();
showHelp() if (!$opt{v} && !$opt{f} || $opt{h});


if ($opt{b})
{
  $pkirisData{'bits'} = $opt{b};
}

# Configuracin de openssl
my $opensslTemplate = "
[ req ]
default_bits            = $pkirisData{'bits'}
default_keyfile         = privkey.pem
distinguished_name      = req_distinguished_name
default_md		= sha1
prompt                  = no
 
[ req_distinguished_name ]
0.DC=es
1.DC=irisgrid
";


if ($opt{f})
{
  # Estamos metiendo otro fichero con los FQDNs
  $pkirisData{'iFQDNsFileName'} = $opt{f};
}
if ($opt{o})
{
  # Especificamos el directorio de salida
  $pkirisData{'oDir'} = $opt{o};
}
if (!(-w $pkirisData{'oDir'} || mkdir($pkirisData{'oDir'}, 0755)))
{
 die "\nERROR: No puedo crear el directorio de salida: $pkirisData{oDir}\n" ;
}
if ($opt{p})
{
  # Especificamos el pin para las CSRs
  $pkirisData{'pin'} = $opt{p};
}
if ($opt{v})
{
  print "\nThis is $pkirisData{myname}, v$pkirisData{version}\n";
  print "(cc) 2006, RedIRIS\n\n";
  exit;
}

  processData ();


#----------------------------------------------------------------------
# Func: processData
# Desc: 
#----------------------------------------------------------------------
sub processData
{

  print "\n";
  print "======================================================================\n";
  print " pkIRISGrid - Generacin de CSRs mltiples + XML\n";
  print " $pkirisData{myname}, v$pkirisData{version}\n";
  print "======================================================================\n\n";
  if ( $opt{d} )
  {
    print "Datos del administrador : ".$pkirisData{iAdminFileName}."\n\n";
    print "  Nombre:    ".$adminData{name}."\n";
    print "  Apellidos: ".$adminData{surname}."\n";
    print "  Clave:     ".$adminData{passwd}."\n";
    print "  Telfono:  ".$adminData{telephone}."\n";
    print "  Mail:      ".$adminData{mail}."\n";
    print "  Dominio:   ".$adminData{domain}."\n";
    print "\n";
    print "Datos de las mquinas/servicios : ".$pkirisData{iFQDNsFileName}."\n";
    print "\n";
  }

  $pkirisXmlData{name}      = "  <name>".$adminData{name}."</name>\n";
  $pkirisXmlData{surname}   = "  <surname>".$adminData{surname}."</surname>\n";
  $pkirisXmlData{passwd}    = "  <passwd>".$adminData{passwd}."</passwd>\n";
  $pkirisXmlData{telephone} = "  <telephone>".$adminData{telephone}."</telephone>\n";
  $pkirisXmlData{mail}      = "  <mail>".$adminData{mail}."</mail>\n";
  $pkirisXmlData{domain}    = "  <domain>".$adminData{domain}."</domain>\n";

  $pkirisXmlData{header} = "<header>\n".$pkirisXmlData{name}.$pkirisXmlData{surname};
  $pkirisXmlData{header} .= $pkirisXmlData{passwd}.$pkirisXmlData{telephone}.$pkirisXmlData{mail};
  $pkirisXmlData{header} .= $pkirisXmlData{domain}."</header>\n";

  # Abrimos el fichero para guardar los PINes de las solicitudes
  $pkirisData{oPinFileName} = $pkirisData{oDir}."/".$pkirisData{oPinFileName};
  open (PIN, ">", $pkirisData{oPinFileName}) ||
       die "\nERROR: No puedo abrir fichero $pkirisData{oPinFileName}\n" ;

  # Abrimos el fichero con los FQDNs
  open (FILE, $pkirisData{iFQDNsFileName}) || 
       die "\nERROR: No puedo abrir fichero $pkirisData{iFQDNsFileName}\n" ;
  while (<FILE>)
  {
    $tmp{cn} = $_;
    # Descartamos las lneas de comentario y las vacas
    if ( ($tmp{cn} !~ "^$pkirisData{iComment}") && ($tmp{cn} ne "\n") )
    {
      chop $tmp{cn};

      # Si tratamos un servicio hay que quitar la "/" del nombre de
      # los ficheros de salida para evitar errores con los directorios
      $tmp{cnTxt} = $tmp{cn};
      $tmp{cnTxt} =~ s/\//$pkirisData{serviceSep}/g if ($tmp{cn} =~ "/");

      # Creamos el fichero de configuracin de openssl
      $tmp{opensslCfName} = $pkirisData{oDir}."/".$pkirisData{opensslCf};
      open (TEMPLATE, ">", $tmp{opensslCfName}) ||
           die "\nERROR: No puedo abrir fichero $tmp{opensslCfName}\n" ;
      print TEMPLATE $opensslTemplate;
      print TEMPLATE "O=".$adminData{domain}."\n" if ($adminData{domain} ne "");
      print TEMPLATE "CN=".$tmp{cn};
      close TEMPLATE;

      # Creamos la cadena con el comando a ejecutar
      $tmp{command} = $pkirisData{openssl}." req -nodes -new \\\n";
      $tmp{command} .= "-config ".$tmp{opensslCfName}." \\\n";
      $tmp{command} .= "-keyout ".$pkirisData{oDir}."/".$tmp{cnTxt}.".key -out ".$pkirisData{oDir}."/".$tmp{cnTxt}.".csr";
      print "\n----------------------------------------------------------------------\n";
      print $tmp{command}."\n" if ($opt{d});

      # Ejecutamos el comando
      system $tmp{command};

      # Cargamos la CSR en una variable para procesarla e introducirla en el XML
      open (CSR, "<", $pkirisData{oDir}."/".$tmp{cnTxt}.".csr") ||
           die "\nERROR: No puedo abrir fichero ".$pkirisData{oDir}."/".$tmp{cnTxt}.".csr\n";
      $tmp{data} = join "", <CSR>;
      close CSR;
     
      # Quitamos cabeceras, fines de linea y pie
      $tmp{data} =~ s/-----BEGIN CERTIFICATE REQUEST-----//g;
      $tmp{data} =~ s/\n//g;
      $tmp{data} =~ s/-----END CERTIFICATE REQUEST-----//g;

      # Obtenemos el PIN de cada solicitud
      if ($pkirisData{pin} eq "random")
      {
        $tmp{pin} = substr(rand(), 2, 6);
      }
      else
      {
        $tmp{pin} = $pkirisData{pin};
      }
      print PIN "$tmp{cn} - $tmp{pin}\n";

      # Llenamos variables de salida
      $pkirisXmlData{csr_cn}   = "    <cn>".$tmp{cn}."</cn>\n";
      $pkirisXmlData{csr_pin}  = "    <pin>".crypt($tmp{pin}, rand(696))."</pin>\n";
      $pkirisXmlData{csr_data} = "    <data>".$tmp{data}."</data>\n";
      $pkirisXmlData{csr} .= "  <csr>\n".$pkirisXmlData{csr_cn}.$pkirisXmlData{csr_pin}.$pkirisXmlData{csr_data}."  </csr>\n";
    }
  }
  # Borramos la configuracin de openssl
  unlink $tmp{opensslCfName};

  # Creamos el fichero XML
  $tmp{oFile} = $pkirisData{oDir}."/".$pkirisData{oCsrXmlFileName};

  print "\n======================================================================\n";
  print " Resultado de la ejecucin:\n\n";

  print " Ficheros con las CSRs geneardas  : ".$pkirisData{oDir}."/*.csr\n";
  print " Ficheros con las claves privadas : ".$pkirisData{oDir}."/*.key\n\n";
  print " Fichero XML con las CSRs: ".$tmp{oFile}."\n";
  print " Fichero con los PINes   : ".$pkirisData{oPinFileName}."\n";

  open (OUTPUT, ">", $tmp{oFile});
    print OUTPUT $pkirisXmlData{top};
    print OUTPUT $pkirisXmlData{header};
    print OUTPUT "<csrs>\n".$pkirisXmlData{csr}."</csrs>\n";
    print OUTPUT $pkirisXmlData{bottom};
  close OUTPUT;

  print "======================================================================\n";

  close FILE;
  close PIN;
}


#----------------------------------------------------------------------
# Func: showHelp
# Desc: Muestra la ayuda
#----------------------------------------------------------------------
sub showHelp
{

  print <<FIN;

$pkirisData{myname} es un programa que genera mltiples CSRs y prepara un 
fichero en formato XML para poder ser cargado en la pkIRISGrid CA

Uso: $pkirisData{myname} [-b bits] [-d] -f FQDN.txt 
                         [-o directorio] [-p pin] [-v] [-h]

 -b Nmero de bits a usar. [$pkirisData{'bits'}]
 -d Debug
 -f Fichero con los FQDNs para los que se solicitan certificados. [$pkirisData{'iFQDNsFileName'}]
 -h Muestra esta ayuda
 -o Directorio de salida. [$pkirisData{'oDir'}]
 -p Pin para las CSRs. [$pkirisData{'pin'}]
    (random: aleatorio, {valor}: ese valor para todos)
 -v Versin del programa. [$pkirisData{'version'}]

Ejemplos

    $pkirisData{myname} -h
    $pkirisData{myname} -f FQDN.txt -o /home/pkIRISGrid/csrs -p kkk
    $pkirisData{myname} -f FQDN.txt -b 1024
    $pkirisData{myname} -v
\n
FIN
;
  exit;
}

#----------------------------------------------------------------------
# Fin
#----------------------------------------------------------------------

